# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import json


def build_influxdb_query(live_query,
                         measurement,
                         group_by=[],
                         order_by=[],
                         additional_filters={},
                         limit=None,
                         multiple_series=False):
    query = ['SELECT * FROM', measurement]

    filters = {}
    time = None
    offset = None
    if live_query:
        if live_query.filters:
            filters.update(json.loads(live_query.filters))
        if live_query.time_interval:
            time = live_query.time_interval
        if live_query.paging:
            if multiple_series:
                limit = live_query.paging.size * (live_query.paging.page + 1)
            else:
                limit = live_query.paging.size
                offset = (live_query.paging.page + 1) * live_query.paging.size

    filters.update(additional_filters)
    query += _build_where_clause(filters, time)

    if group_by:
        query.append('GROUP BY')
        query.append(', '.join(group_by))

    if order_by:
        query.append('ORDER BY')
        query.append(', '.join(order_by))

    if limit is not None:
        query.append('LIMIT %d' % limit)

    if offset is not None:
        query.append('OFFSET %d' % offset)

    return ' '.join(query)


def _build_where_clause(filters, time=None):
    filters_conversion = {
        'is': '=',
        'isnot': '!='
    }
    clause = []
    is_where_append = False

    if time:
        clause.append('WHERE')
        clause.append("time >= '%s' AND time <= '%s'" %
                      (time.start_time, time.end_time))
        is_where_append = True

    for filter_name, filter_data in sorted(filters.items()):
        for field, values in sorted(filter_data.items()):
            for value in values:
                if not is_where_append:
                    clause.append('WHERE')
                    is_where_append = True
                else:
                    clause.append('AND')

                if type(value) == int:
                    clause.append("%s%s%d" % (field,
                                              filters_conversion[filter_name],
                                              value))
                else:
                    clause.append("%s%s'%s'" %
                                  (field,
                                   filters_conversion[filter_name],
                                   value))

    return clause


def paging(response, datamodel, live_query=None):
    """Paging function

    :param response: a python-influxdb resulset
    :param datamodel: an Surveil API datamodel class
    :param live_query: an influxdb_query
    :return: a dict of datamodel object. If the live query contain paging,
    the dict is sorted by datamodel time attribute and contain
    live_query.paging.size object for the live_query.paging.page page
    """
    if live_query and live_query.paging:
        limit_paging = live_query.paging.size * (live_query.paging.page + 1)
        limit = limit_paging + live_query.paging.size
        offset_paging = live_query.paging.page * live_query.paging.size

        def sort_by_time(init, point_tag):
            event = {}
            event.update(point_tag[0])
            event.update(point_tag[1])
            init.append(datamodel(**event))
            init.sort(key=lambda event: event.time,
                      reverse=True)
            return init[:limit_paging]

        response = [(tag[1], _dict_from_influx_item(datamodel, point))
                    for tag, points in response.items()
                    for point in points]

        event_list = functools.reduce(sort_by_time, response, [])

        return event_list[offset_paging:limit+1]

    else:
        events = []

        for item in response.items():
            tags = item[0][1]
            for point in response.get_points(tags=tags):
                point.update(tags)
                event_dict = _dict_from_influx_item(datamodel, point)
                events.append(datamodel(**event_dict))

        return events


def _dict_from_influx_item(datamodel, item):
    """Create a dict representing a python-influxdb item

    :param item: an python influxdb item object
    :param datamodel: an Surveil API datamodel class
    :return: a dict (datamodel_attribute:item_value)

    >>> _event_dict_from_influx_item(Event, {"time": 4})
    {'time': 4}
    >>> _event_dict_from_influx_item(Event, {"time": "null"})
    {}
    """

    fields = [attr.name for attr in getattr(datamodel, "_wsme_attributes")]
    return dict([(field, item.get(field, None)) for field in fields
                 if item.get(field, None) is not None])



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import mongoengine

from surveil.api.storage.mongodb import foreign_key_field


def build_mongoengine_query(live_query, resource_storage):

    query = mongoengine.Q()

    # Filters
    if live_query.filters and json.loads(live_query.filters).items():
        for filter_name, filter_data in json.loads(live_query.filters).items():
            for field, value in filter_data.items():
                qobj = mongoengine.Q(
                    **_get_mongoengine_filter(field,
                                              filter_name,
                                              value)
                )
                query = query & qobj

    # search
    if live_query.search:
        search_q = None

        string_fields = [
            field for field in resource_storage._fields
            if isinstance(
                getattr(resource_storage, field),
                (
                    mongoengine.StringField,
                    mongoengine.ListField,
                    foreign_key_field.ForeignKeyListField,
                    foreign_key_field.ForeignKeyListField
                )
            )
        ]

        for field in string_fields:
            field_q = mongoengine.Q(
                __raw__={
                    field: {"$regex": ".*%s.*" % live_query.search,
                            "$options": "-i"}
                }
            )

            if search_q is None:
                search_q = field_q
            else:
                search_q = search_q | field_q

        query = query & search_q

    # Fields
    fields = []
    if live_query.fields:
        for field in live_query.fields:
            fields.append(field)

    # Paging
    skip = None
    limit = None
    if live_query.paging:
        skip = live_query.paging.size * live_query.paging.page
        limit = skip + live_query.paging.size

    return fields, query, skip, limit


def _get_mongoengine_filter(field_name, filter_name, value):
    filters = {
        "is": field_name + "__in",
        "isnot": field_name + "__nin",
        "defined": field_name + "__exists"
    }
    return {filters[filter_name]: value}


# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from surveil.api.datamodel import live_query
from surveil.api.datamodel.status import paging
from surveil.api.handlers.config import mongoengine_query
from surveil.api.storage.mongodb.config import host
from surveil.tests import base


class MongoEngineliveQueryTest(base.BaseTestCase):

    def test_build_mongoengine_query(self):
        query = live_query.LiveQuery(
            fields=['host_name', 'last_check'],
            filters=json.dumps({
                "isnot": {
                    "state": ["0", "1"],
                    "host_state": ["2"]
                },
                "is": {
                    "event_type": ["ALERT"]
                },
                "defined": {
                    "name": True
                }
            }),
            paging=paging.Paging(
                page=3,
                size=100
            )

        )

        fields, query, skip, limit = mongoengine_query.build_mongoengine_query(
            query,
            host.Host
        )

        self.assertEqual(
            fields,
            ['host_name', 'last_check']
        )

        self.assertEqual(skip, 300)
        self.assertEqual(limit, 400)



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import time

from compose.cli import docker_client
from compose import config as compose_config
from compose import project as compose_project
import docker
from surveilclient import client as sclient


class DockerBackend():

    def __init__(self):
        pass

    def setUpClass(self):
        surveil_dir = os.path.realpath(
            os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                "../../../../"
            )
        )

        compose_file = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'integration.yml'
        )

        project_config = compose_config.from_dictionary(
            compose_config.load_yaml(compose_file),
            working_dir=surveil_dir,
            filename=compose_file
        )

        self.project = compose_project.Project.from_dicts(
            "surveilintegrationtest",
            project_config,
            docker_client.docker_client()
        )

        self.project.kill()
        self.project.remove_stopped()
        self.project.build()
        self.project.up()

        self.surveil_client = sclient.Client(
            'http://localhost:8999/v2',
            auth_url='http://localhost:8999/v2/auth',
            version='2_0'
        )

        #  Wait until Surveil is available
        now = time.time()
        while True:
            print("Waiting for surveil... %s" % int(time.time() - now))
            if time.time() < (now + 380):
                try:
                    #  If 'ws-arbiter' is found, Surveil is ready!
                    configured_hosts = self.surveil_client.status.hosts.list()
                    host_found = False
                    for host in configured_hosts:
                        if host['host_name'].decode() == 'ws-arbiter':
                            host_found = True
                            break
                    if host_found:
                        break
                except Exception:
                    pass
                time.sleep(10)
            else:
                raise Exception("Surveil could not start")

    def execute_command(self, commands, container_name):
        dclient = docker.Client()
        if container_name is not None:
            containers = dclient.containers()
            for container in containers:
                count = container.get('Image').count(container_name)
                if ((count == 1 and container_name != 'surveil') or
                        (count == 2 and container_name == 'surveil')):
                    for command in commands:
                        id = dclient.exec_create(container.get('Id')[:12],
                                                 command)
                        dclient.exec_start(id['Id'])

    def tearDownClass(self):
        self.project.kill()
        self.project.remove_stopped()

    def get_surveil_client(self):
        return self.surveil_client



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json


def build_mongodb_query(live_query):
    query = []
    kwargs = {}

    #  Build the filters
    filters = {}
    for filter_name, filter_data in live_query.get('filters', {}).items():
        for field, values in filter_data.items():
            filters[field] = {
                _get_mongo_filter(filter_name): values
            }

    search = live_query.get('search', None)
    if search:
        filters["$text"] = {"$search": search}

    if filters:
        query.append(filters)

    #  Build the required fields
    fields = {}
    for field in live_query.get("fields", []):
        fields[field] = 1

    if fields:
        query.append(fields)

    # Paging
    paging = live_query.get('paging', None)
    if paging is not None:
        kwargs['limit'] = paging.size
        kwargs['skip'] = paging.size * paging.page

    return query, kwargs


def _get_mongo_filter(livequery_filter):
    filters = {
        "is": "$in",
        "isnot": "$nin"
    }
    return filters[livequery_filter]


def translate_live_query(live_query, mappings):
    """Translate field names in a live query so that they match mongodb."""

    #  Load the fields
    fields = live_query.get("fields", [])

    #  Translate the fields
    translated_fields = []
    for field in fields:
        translated_fields.append(mappings.get(field, field))
    live_query["fields"] = translated_fields

    #  Load the filters
    filters = json.loads(live_query.get("filters", '{}'))

    #  Translate the filters
    for filter in filters.values():
        for field in filter.keys():
            value = filter.pop(field)
            filter[mappings.get(field, field)] = value
    live_query["filters"] = filters

    return live_query


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import pecan
from pecan import rest


class TokensController(rest.RestController):

    @pecan.expose()
    def post(self):
        """Retrieve an auth token."""

        access = {
            "access": {
                "token": {
                    "issued_at": "2014-01-30T15:30:58.819584",
                    "expires": "2014-01-31T15:30:58Z",
                    "id": "aaaaa-bbbbb-ccccc-dddd",
                    "tenant": {
                        "description": "Hey!",
                        "enabled": True,
                        "id": "fc394f2ab2df4114bde39905f800dc57",
                        "name": "demo"
                    }
                }
            }
        }
        return json.dumps(access)



# Copyright 2015 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import wsme
import wsme.types as wtypes

from surveil.api.datamodel import types


class Event(types.Base):

    time = wsme.wsattr(wtypes.text, mandatory=True)
    """Timestamp of the alert"""

    event_type = wsme.wsattr(wtypes.text, mandatory=True)
    """Type of event. This is only ALERT"""

    host_name = wsme.wsattr(wtypes.text, mandatory=False)
    """Host which the alert is from."""

    service_description = wsme.wsattr(wtypes.text, mandatory=False)
    """Service which raised the alert"""

    state = wsme.wsattr(wtypes.text, mandatory=False)
    """State of the service or host who raised the alert"""

    # Alerts
    state_type = wsme.wsattr(wtypes.text, mandatory=False)
    """Confirmness level of the state [SOFT|HARD]"""

    attempts = wsme.wsattr(int, mandatory=False)
    """Number of attempts to confirm state"""

    # Downtime
    downtime_type = wsme.wsattr(wtypes.text, mandatory=False)
    """Type of alert. This is only HOST or SERVICE"""

    # Notifications
    notification_type = wsme.wsattr(wtypes.text, mandatory=False)

    notification_method = wsme.wsattr(wtypes.text, mandatory=False)

    contact = wsme.wsattr(wtypes.text, mandatory=False)

    acknowledgement = wsme.wsattr(wtypes.text, mandatory=False)

    # Alert, Flapping
    alert_type = wsme.wsattr(wtypes.text, mandatory=False)
    """Type of alert. This is only HOST or SERVICE"""

    # Alerts, Downtime, Flapping
    output = wsme.wsattr(wtypes.text, mandatory=False)
    """Additional output of the alert."""

    @classmethod
    def sample(cls):
        return cls(
            time='2015-06-04T18:55:12Z',
            event_type='ALERT',
            alert_type='SERVICE',
            host_name='CoolHost',
            service_description='Apache Service',
            state='CRITICAL',
            state_type='HARD',
            attempts=4,
            output='WARNING - load average: 5.04, 4.67, 5.04',
            notification_method='notify-service-by-email',
            notification_type=''
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests

from surveil.tests.integration import integration_test


class TestMergedIngegrationSurveil(
    integration_test.MergedIntegrationTest
):
    def test_hello(self):
        self.assertEqual(
            requests.get("http://localhost:8999/v2/hello").text,
            'Hello World!'
        )


class TestSeparatedIntegrationSurveil(
    integration_test.SeparatedIntegrationTests
):
    def test_create_host(self):
        """Creates a host and asserts that is is monitored by Alignak."""
        config_hosts = self.get_surveil_client().status.hosts.list()

        self.assertFalse(
            any(host['host_name'] == 'integrationhosttest'
                for host in config_hosts)
        )

        self.get_surveil_client().config.hosts.create(
            host_name='integrationhosttest',
            address='127.0.0.1',
            use=['generic-host'],
        )

        self.get_surveil_client().config.reload_config()

        def function():
            status_hosts = self.get_surveil_client().status.hosts.list()
            self.assertTrue(
                any(host['host_name'].decode() == 'integrationhosttest'
                    for host in status_hosts)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Could not find host in status."
            )
        )

    def test_delete_host(self):
        """Delete a host and asserts that is is not monitored by Alignak."""
        self.test_create_host()

        self.get_surveil_client().config.hosts.delete(
            'integrationhosttest'
        )

        self.get_surveil_client().config.reload_config()

        def function():
            status_hosts = (self.get_surveil_client().status.hosts.list())
            self.assertFalse(
                any(host['host_name'].decode() == 'integrationhosttest'
                    for host in status_hosts)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Host was not deleted"
            )
        )

    def test_update_host(self):
        """Update a host and asserts that is is monitored by Alignak."""
        self.test_create_host()

        self.get_surveil_client().config.hosts.update(
            host_name='integrationhosttest',
            host={'host_name': 'host_name_up',
                  'address': '127.0.1.1'}
        )

        self.get_surveil_client().config.reload_config()

        def function():
            status_host = (self.get_surveil_client().
                           config.hosts.get(host_name='host_name_up'))
            use = []
            for status_host_use in status_host['use']:
                use.append(status_host_use.decode())

            self.assertTrue(
                status_host['host_name'].decode() == 'host_name_up' and
                status_host['address'].decode() == '127.0.1.1' and
                use == ['generic-host']
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Host is not updated."
            )
        )

    def test_passive_check(self):
        """Test  monitoring a host with passive checks."""
        self.test_create_host()

        self.get_surveil_client().config.services.create(
            check_command="check_integrationhosttest",
            check_interval="5",
            check_period="24x7",
            contact_groups=["admins"],
            contacts=["admin"],
            host_name=["integrationhosttest"],
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="check_integrationhosttest",
            passive_checks_enabled="1"
        )

        self.get_surveil_client().config.reload_config()
        self.get_surveil_client().status.services.submit_check_result(
            host_name=['integrationhosttest'],
            service_description='check_integrationhosttest',
            output="Hello",
            return_code=0
        )

        def get_host_name(service):
            host_name_list = []
            for host_name in service['host_name']:
                host_name_list.append(host_name.decode())
            return host_name_list

        def function():
            status_services = self.get_surveil_client().status.services.list()
            self.assertFalse(
                any(get_host_name(service) == ['integrationhosttest'] and
                    service['service_description'].decode() ==
                    'check_integrationhosttest' and
                    service['plugin_output'].decode() == 'Hello' and
                    service['state'].decode() == 'OK'
                    for service in status_services)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="submit check result fail"
            )
        )

    def test_custom_plugins(self):
        """Test documentation tutorial monitoring with your custom plugins."""
        commands = [
            "mkdir /usr/lib/monitoring/plugins/custom/",
            "echo -e '#!/bin/bash\necho " +
            "DISK $1 OK - free space: / 3326 MB (56%);"
            " | /=2643MB;5948;5958;0;5968" +
            "' | sudo tee /usr/lib/monitoring/plugins/"
            "custom/check_example",
            'chmod +x /usr/lib/monitoring/plugins/custom/'
            'check_example'
        ]

        self.execute_command(commands, 'alignak')

        self.test_create_host()
        self.get_surveil_client().config.commands.create(
            command_name='check_integrationhosttest',
            command_line='$CUSTOMPLUGINSDIR$/check_example $HOSTADDRESS$'
        )
        self.get_surveil_client().config.services.create(
            check_command="check_integrationhosttest",
            check_interval="5",
            check_period="24x7",
            contact_groups=["admins"],
            contacts=["admin"],
            host_name=["integrationhosttest"],
            max_check_attempts="5",
            notification_interval="30",
            notification_period="24x7",
            retry_interval="3",
            service_description="check_integrationhosttest"
        )

        self.get_surveil_client().config.reload_config()

        def function():
            status_services = self.get_surveil_client().status.services.list()
            self.assertFalse(
                any(service['host_name'].decode() == 'integrationhosttest' and
                    service['service_description'].decode() ==
                    'check_integrationhosttest' and
                    service['plugin_output'].decode() ==
                    "DISK 127.0.1.1 OK - free space: / 3326 MB (56%);"
                    "| /=2643MB;5948;5958;0;5968"
                    for service in status_services)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=180,
                cooldown=10,
                exception=AssertionError,
                message="Custom Plugins is not used"
            )
        )

    def test_list_metrics_name_host(self):
        """Test if a host with use=generic-host have the good metrics name."""
        self.test_create_host()

        def function():
            metrics_name_hosts = (
                self.get_surveil_client().status.hosts.metrics.get(
                    host_name='integrationhosttest')
            )
            self.assertTrue(
                any(metric_name['metric_name'].decode() == 'rtmin'
                    for metric_name in metrics_name_hosts)
            )

        self.assertTrue(
            self.try_for_x_seconds(
                function,
                time_to_wait=360,
                cooldown=10,
                exception=AssertionError,
                message="No metric name for host created"
            )
        )


# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import json


from surveil.api.datamodel.config import realm
from surveil.tests.api import functionalTest


class TestRealmsController(functionalTest.FunctionalTest):

    def setUp(self):
        super(TestRealmsController, self).setUp()
        self.realms = [
            {
                'realm_name': 'World',
                'realm_members': ['Europe', 'America', 'Asia'],
                'default': 0
            },
            {
                'realm_name': 'Anti-world',
                'realm_members': ['void', 'black-hole'],
                'default': 1
            },
        ]
        self.mongoconnection.shinken.realms.insert(
            copy.deepcopy(self.realms)
        )

    def test_get_all_realms(self):
        response = self.post_json('/v2/config/realms', params={})

        self.assert_count_equal_backport(
            json.loads(response.body.decode()),
            self.realms
        )
        self.assertEqual(response.status_int, 200)

    def test_get_one_realm(self):
        response = self.get('/v2/config/realms/World')

        self.assertEqual(
            json.loads(response.body.decode()),
            self.realms[0]
        )

    def test_create_realm(self):
        r = realm.Realm(
            realm_name='John',
            realm_members=['marie', 'bob', 'joe'],
            default=1
        )

        self.put_json('/v2/config/realms', r.as_dict())

        self.assertIsNotNone(
            self.mongoconnection.shinken.realms.find_one(r.as_dict())
        )

    def test_delete_realm(self):
        self.assertIsNotNone(
            self.mongoconnection.shinken.realms.find_one(self.realms[0])
        )

        self.delete('/v2/config/realms/World')

        self.assertIsNone(
            self.mongoconnection.shinken.realms.find_one(self.realms[0])
        )

    def test_put_realm(self):
        self.assertEqual(
            self.mongoconnection.shinken.realms.find_one(
                {'realm_name': 'World'}
            )['realm_members'],
            ['Europe', 'America', 'Asia']
        )

        self.put_json(
            '/v2/config/realms/World',
            {"realm_name": "World",
             "realm_members": ["updated"],
             "default": 0}
        )

        self.assertEqual(
            self.mongoconnection.shinken.realms.find_one(
                {'realm_name': 'World'}
            )['realm_members'],
            ['updated']
        )



# Copyright 2014 - Savoir-Faire Linux inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mongoengine

from surveil.api.storage.mongodb.config import service
from surveil.api.storage.mongodb import foreign_key_field


class ServiceGroup(mongoengine.Document):
    meta = {'collection': 'servicegroups'}
    servicegroup_name = mongoengine.StringField(unique=True)
    members = foreign_key_field.ForeignKeyListField(
        service.Service,
        ['service_description', 'name']
    )
    alias = mongoengine.StringField()
    servicegroup_members = mongoengine.ListField()
    notes = mongoengine.StringField()
    notes_url = mongoengine.StringField()
    action_url = mongoengine.StringField()
